﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using Ververg;

namespace Ververg
{
    public class InfinitySpawner : MonoBehaviour {

        [Tooltip("Initial object where we spawn next objects next to this initial object.")]
        public GameObject InitialObject;
        [Tooltip("Variant sprites that will randomly used when starting the game")]
        public Sprite[] Sprites;
        [Tooltip("Max pooled object. use the minimum count (the lowest that doesnt cut in the game) for higher perfomance")]
        [Range(1, 5)]
        public int MaxObject = 3;
        public bool isHorizontal = true;
        public bool useScale = false;
        public float substractPos = 0;

        private GameObject _CurrentObject;
        private List<GameObject> _ObjectPools = new List<GameObject>();
        private int _PoolIndex = 0;

        void Start () {
            // Initiated pooling
            _CurrentObject = InitialObject;
            _ObjectPools.Add(_CurrentObject);

            // Initiate max object
            for (int i = 0; i < MaxObject - 1; i++)
            {
                SpawnNext();
            }
        }

        // Spawn the next object after the last one
        public void SpawnNext()
        {
            Vector3 nextPost;
            if(isHorizontal)
                nextPost = new Vector3(_CurrentObject.transform.position.x + _CurrentObject.GetComponent<BoxCollider2D>().size.x  - substractPos, _CurrentObject.transform.position.y, _CurrentObject.transform.position.z);
            else {
                if(!useScale)
                    nextPost = new Vector3(_CurrentObject.transform.position.x, _CurrentObject.transform.position.y + _CurrentObject.GetComponent<BoxCollider2D>().size.y - substractPos, _CurrentObject.transform.position.z);
                else
                    nextPost = new Vector3(_CurrentObject.transform.position.x, _CurrentObject.transform.position.y + _CurrentObject.GetComponent<BoxCollider2D>().size.y + _CurrentObject.transform.localScale.y / 2  - substractPos, _CurrentObject.transform.position.z);
            }
                
            if(_ObjectPools.Count == MaxObject)
            {
                _CurrentObject = _ObjectPools[_PoolIndex];
                _CurrentObject.transform.position = nextPost;
                NextPool();
            }
            else
            {
                _CurrentObject = Instantiate(InitialObject, nextPost, InitialObject.transform.rotation) as GameObject;
                _CurrentObject.transform.parent = InitialObject.transform.parent;
                _ObjectPools.Add(_CurrentObject);
            }
        }

        // Pooling
        void NextPool()
        {
            _PoolIndex++;
            if(_PoolIndex >= MaxObject)
            {
                _PoolIndex = 0;
            }
        }

        public List<GameObject> GetObjectsPool() {
            return _ObjectPools;
        }
    }
}